﻿using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;
using System.Collections;
using System.Linq;

namespace SimpleGrid
{
	public partial class SimpleGridViewController : UIViewController
	{
		#region Members

		IGGridView gridView;

		customHelper dsh;

		List<cSimpleGrid> lstGridItems;


        #endregion

        #region Constructor

        public SimpleGridViewController () : base ("SimpleGridViewController", null)
		{
		}

		#endregion

		#region View Controller Methods

		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();


            if (UIDevice.CurrentDevice.CheckSystemVersion(7, 0))
            {

                this.EdgesForExtendedLayout = UIRectEdge.None;
                this.AutomaticallyAdjustsScrollViewInsets = false;
            }


            //Next, create a new instance of the grid view and place it on the view.
            gridView = new IGGridView (new System.Drawing.RectangleF (0, 0, this.View.Bounds.Width, this.View.Bounds.Height), IGGridViewStyle.IGGridViewStyleDefault);
			gridView.AutoresizingMask = UIViewAutoresizing.All;
		    gridView.HeaderHeight = 0;
            gridView.SelectionType = IGGridViewSelectionType.IGGridViewSelectionTypeNone;
            
			this.View.Add (gridView);

         

		    //Setup the Grid
			SetupGrid ();
		}

		#endregion

		#region Methods

		/// <summary>
		/// This method is responsible for the layour and populate of the grid view
		/// </summary>
		private bool SetupGrid ()
		{
			//First, create a new return variable
			bool bReturn = true;

			try {

				// We're using the basic IGGridViewDataSourceHelper
				// since our data will be displayed in a standard datagrid
				dsh = new customHelper();

				// Since we are manually creating columns, turn off auto-generation
				dsh.AutoGenerateColumns = false;


                customGoalsColumnDefinition type1 = new customGoalsColumnDefinition("PT", this);
                customGoalsColumnDefinition type2 = new customGoalsColumnDefinition("OT", this);
                customGoalsColumnDefinition type3 = new customGoalsColumnDefinition("ST", this);

                dsh.ColumnDefinitions.Add(type1);
                //dsh.ColumnDefinitions.Add(type2);
                //dsh.ColumnDefinitions.Add(type3);


                dsh.AutoGenerateColumns = true;

                dsh.AllowEditing = true;

				//Set the DataSource
				gridView.DataSource = dsh;

				//Turn off empty rows
				gridView.EmptyRows = false;

				//Update gridView
				gridView.UpdateData ();

			} catch {
				//Error building the grid, return false.
				bReturn = false;
			}

			//Finally, return the return variable
			return bReturn;
		}

		#endregion

		public class customGoalsColumnDefinition : IGGridViewColumnDefinition
		{
			#region Members

			string _fieldKey;
			SimpleGridViewController mController;

			#endregion

			#region Constructor

			public customGoalsColumnDefinition (string key, SimpleGridViewController controller) : base ()
			{
				_fieldKey = key;
				mController = controller;

			}

			#endregion

			#region Properties

			public override string FieldKey {
				get {
					return _fieldKey;
				}
			}

		    public override bool Editable 
		    {
				get {
					return true;
				}
			}

			#endregion

			#region Override Methods


        
            

			public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
			{
                IGGridViewCell cell = (IGGridViewCell)gridView.DequeueReusableCell("Cell");

                if (cell == null)
                {
                    cell = new IGGridViewCell("Cell");
                }



                Console.WriteLine($"old = {cell.TextLabel.Text} new = {_fieldKey}.{path.ColumnIndex}");

                cell.TextLabel.Text = $"{_fieldKey}.{path.ColumnIndex}";

                return cell;
			}


		    public override void DisplayEditor(IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
		    {
		        base.DisplayEditor(gridView, path, dataSource);
		    }

		    public override void RemoveEditor (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource, bool cellVisible, bool anotherCellEnteringEditMode)
			{
				gridView.UnregisterKeyboard (!anotherCellEnteringEditMode);

				IGGridViewCell cell = (IGGridViewCell)gridView.ResolveCellForPath (path);

		        if (cell.Subviews.Length > 0 && cell.Subviews[0] is UITextField)
		        {
		            UITextField field = (UITextField) cell.Subviews[0];
		            cell.TextLabel.Text = field.Text;

		            float duration = cellVisible ? dataSource.EditingTransitionDuration : 0.0f;

				    UIView.Transition (field, cell.TextLabel, duration, UIViewAnimationOptions.TransitionFlipFromTop, null);
                }

                //Before returning the cell, apply the styling to the cell
                //ApplyStylingToCell(cell, path.RowIndex, gridView);

				//THIS IS THE LINE OF CODE THAT I USE WHEN A USER POPULATES THE BLANK ROW, TO AUTO RE-ADD A NEW BLANK ROW AT THE END, SO THERE IS ALWAYS A BLANK ROW.
				//gridView.UpdateData ();

			}

			#endregion

		}

		public class customHelper : IGGridViewDataSourceHelper
		{
            private IGGridViewCell _editCell;

            public override IGCellPath NormalizePath(IGCellPath path)
            {
                return path;
                return new IGCellPath(path.ColumnIndex, path.SectionIndex, path.RowIndex);
            }


            public override IGGridViewCell CreateCell(IGGridView gridView, IGCellPath path)
            {

                IGCellPath normPath = this.NormalizePath(path);

 
                var colDef = this.ColumnDefinitions.GetItem<NSObject>(normPath.RowIndex) as IGGridViewColumnDefinition;

                if (colDef != null)
                {
                    IGGridViewCell cell = colDef.CreateCell(gridView, path, this);

                    if (this.AllowEditing && colDef.Editable)
                    {
                        var tapGesture = new UITapGestureRecognizer(this, new MonoTouch.ObjCRuntime.Selector("HandleEdit:"))
                        {
                            NumberOfTapsRequired = 1
                        };

                        cell.RegisterGestures(new NSObject[] { tapGesture });


                    }


                    return cell;
                }

                return null;
            }

            [Export("HandleEdit:")]
            private void HandleEdit(UITapGestureRecognizer gestureRecognizer)
            {

                gestureRecognizer.Reset();

                var cell = gestureRecognizer.View as IGGridViewCell;
                if (cell == null)
                    return;

                if (_editCell != null && cell != _editCell)
                {
                    IGCellPath lastPath = this.NormalizePath(_editCell.Path);

                    var lastCol = this.ColumnDefinitions.GetItem<NSObject>(lastPath.RowIndex) as IGGridViewColumnDefinition;
                    lastCol.RemoveEditor(_editCell.GridView, _editCell.Path, this, true, true);
                }

                _editCell = cell;


                IGCellPath normPath = this.NormalizePath(cell.Path);

                var colDef = this.ColumnDefinitions.GetItem<NSObject>(normPath.RowIndex) as IGGridViewColumnDefinition;
                colDef.DisplayEditor(cell.GridView, cell.Path, this);

    

            }



            public override void DidEndDisplayingCell(IGGridView gridView, IGGridViewCell cell, IGCellPath path)
            {
                if (cell == _editCell)
                {
                    IGCellPath normPath = this.NormalizePath(cell.Path);

                    var colDef = this.ColumnDefinitions.GetItem<NSObject>(normPath.RowIndex) as IGGridViewColumnDefinition;
                    colDef.RemoveEditor(cell.GridView, cell.Path, this, false, false);
                }
            }


            public override int NumberOfRowsInSection(IGGridView gridView, int section)
            {
                return (int)this.ColumnDefinitions.Count;
            }


            public override int NumberOfColumns(IGGridView gridView)
            {
                return 10;
            }

            public override bool ProvideRowHeights(IGGridView gridView)
            {
                return true;
            }

            public override float ResolveRowHeight(IGGridView gridView, IGRowPath path)
            {
                return 40;
            }

            public override IGColumnWidth ResolveColumnWidth(IGGridView gridView, int nint)
            {
                return IGColumnWidth.CreateNumericColumnWidth(180);

            }
            public override int NumberOfSections(IGGridView gridView)
            {
                return 1;
            }



        }

	
	}

  
}

